#ifndef CFNT_FONT_INC_PRG
#define CFNT_FONT_INC_PRG

/**
 *  Custom Font System for BennuGD
 * 
 *  EugeneP (popov_e_n@mail.ru), 2010
 * 
 *  .psf fonts and .enc files can be found
 *  on virtually any *nix system in /usr/share/fonts
 * 
 **/
import "mod_file"
import "mod_say"
import "mod_map"
import "mod_draw"
import "mod_string"

#include "../lib/sysopts.inc.prg"

#define dig_hex2int(dig)  ( dig>="A"?(10+asc(dig)-asc("A")):(asc(dig)-asc("0")) )

/*---------------------------------------------------------------------------------*/

#define MAX_CFNT_GLYPHS 512
#define MAX_CFNT_GLYPH_BWIDTH 1 /* in bytes  3 = 24bit */
#define MAX_CFNT_GLYPH_HEIGHT 25 /* in lines */

/*---------------------------------------------------------------------------------*/

type CFNT_FONT
  int native_analog = 0;
  
/// monospaced raster font:
  int width, height; /* glyph size in pixels/bits */
  int length; /* number of glyphs */
  int asciifont; /* 8bit ascii font (unicode map is not used) */
  byte glyph[ MAX_CFNT_GLYPHS ][MAX_CFNT_GLYPH_HEIGHT] /*[ MAX_CFNT_GLYPH_WIDTH ]*/;
  int  glmap[ MAX_CFNT_GLYPHS ]; // prerendered character glyphs
  
  word uni2glyph[ MAX_CFNT_GLYPHS ]; /* map unicode 2 glyph */
  word encoding[ MAX_CFNT_GLYPHS ]; // map ascii -> unicode
  
  word map_uni[ 65536 ]; // search index unicode -> glyph
  word map_asc[ 256 ];   // search index ascii   -> glyph
end;

/*---------------------------------------------------------------------------------*/

function int cfnt_add_encoding_from_file( CFNT_FONT *afont, string encfile )
local
  int fd, i,j,k, ac, uc, t;
  string b1, b2;
end
begin
  

  fd = fopen( encfile, "rt" );
  if( !fd ) say("Not found: "+encfile); return -1; end
  
  k=0;
  while(!feof(fd))

    loop
      b1 = trim(fgets(fd));
      if( feof(fd) || b1=="STARTMAPPING unicode" ) break; end;
    end;

    loop
      b1 = trim(fgets(fd));
      if( feof(fd) || b1=="ENDMAPPING" ) break; end;

      i = find( b1, "0x", 0);
      if(i<0) continue; end;
      j = find( b1, "0x", i+4);
      if(j<0) continue; end;

      ac = dig_hex2int(b1[i+2]) * 16 + dig_hex2int(b1[i+3]);
      uc = dig_hex2int(b1[j+2])* 16 * 16 * 16 + dig_hex2int(b1[j+3]) * 16 * 16 + dig_hex2int(b1[j+4]) * 16 + dig_hex2int(b1[j+5]);

      afont.encoding[ac] = uc;
    end;
    
  end

  fclose(fd);
  return 0;
end

/*---------------------------------------------------------------------------------*/

/// returns a pointer to glyph or null;
/// for ascii fonts uni_code is threated as ascii code
function int cfnt_unicode(CFNT_FONT *afont, int uni_code)
local
  int i;
end
begin

  if( afont.asciifont )
    if( uni_code>=0 &&  uni_code<=255 )
      return uni_code;
    end;
    return -1;
  end;

  return afont.map_uni[ uni_code ];

/*
  for( i=0; i<afont.length; i++)
    if( uni_code == afont.uni2glyph[i] )
      //say("glyph "+i);
      return i;
    end;
  end;

  return -1;
*/
end;

/*---------------------------------------------------------------------------------*/

function int cfnt_ascii(CFNT_FONT *afont, int ascii)
begin
  if( afont.asciifont )
    //return cfnt_unicode( afont, ascii );
    return afont.map_uni[ ascii ];
  else
    //say("ascii "+ascii+" -> u"+afont.encoding[ ascii ]);
    return cfnt_unicode( afont, afont.encoding[ ascii ] );
    //return afont.map_uni[ afont.encoding[ ascii ] ];
  end;
end;

/*---------------------------------------------------------------------------------*/
// 8-bit char write, encoding of text must be the same as encoding of font 
function cfnt_write_to_map( CFNT_FONT *font, int map, int x, int y, string text )
local
    int g, i, l;
end
begin

  l = len( text );  
  for( i=0; i<l; i++ )
    g = cfnt_ascii( font, asc( text[i] ) );
    if( g >= 0 )
      map_put( file, map, font.glmap[ g ] , x, y );
      //map_unload( file, g );
    else
      say("No char in font: "+asc( text[i] ) );
    end;
    x += font.width;
  end;
    
end

/*---------------------------------------------------------------------------------*/
// Whide char write (char = 16bit unicode), practicaly it's equal to UCS2 encoding
function cfnt_wcwrite_to_map( CFNT_FONT *font, int map, int x, int y, word *wctext )
local
    int g, i;
end
begin
  
  for( i=0; wctext[i] !=  0; i++ )
    g = cfnt_unicode( font, wctext[i] );
    if( g >= 0 )
      map_put( file, map, font.glmap[ g ] , x, y );
      //map_unload( file, g );
    else
      say("No wchar in font: "+ wctext[i] );
    end;
    x += font.width;
  end;
    
end

/*---------------------------------------------------------------------------------*/

function cfnt_reindex ( CFNT_FONT *font )
local
  int i, j, b, map;
end
begin

  if( font.asciifont )
    return;
  end;

  for( i = 0; i < 65536; i++ )
    font.map_uni[i] = -1;
  end;

  for( i = 0; i < font.length; i++ )
    font.map_uni[ font.uni2glyph[i] ] = i;
  end;

end;


/*---------------------------------------------------------------------------------*/

function cfnt_prerender_maps( CFNT_FONT *font )
local
  int i, j, b, map;
  byte *gl;
end
begin
  for( i=0; i<font.length; i++ )
    map = map_new( font.width, font.height, /*sys_default_bpp()*/ 16 );
    set_center( file, map, 0, 0 );
    gl = &font.glyph[ i ];
    for( j = 0; j < font.height; j++ )
      for( b = font.width; b>=0 ; b-- )
	if( gl[ j ] & ( 1 << b ) & 255 )
	  map_put_pixel ( file , map , font.width-b, j , rgb(255, 255, 255) );
	end;
      end;
    end;
    font.glmap [ i ] = map;
  end
end;

/*---------------------------------------------------------------------------------*/
function cfnt_unload( CFNT_FONT *font )
local 
  int i;
end
begin
  for( i=0; i<MAX_CFNT_GLYPHS; i++ )
    if ( font.glmap[i] )
      map_unload(file, font.glmap[i] );
    end;
  end;
end;

/*---------------------------------------------------------------------------------*/

#include "../lib/cfnt_font_psf.inc.prg"

#endif