/**
 *  Custom Font System for BennuGD
 * 
 *  EugeneP (popov_e_n@mail.ru), 2010
 * 
 *  Supports PSF1 console fonts.
 * 
 **/

import "mod_file"
import "mod_say"
       
/// copypasted mostly from linux mans

//For psf1 this is
#define PSF1_MAGIC0     54 //36h
#define PSF1_MAGIC1     4 //04h

#define PSF1_MODE512    1 //01h
#define PSF1_MODEHASTAB 2 //02h
#define PSF1_MODEHASSEQ 3 //04h
#define PSF1_MAXMODE    4 //05h

#define PSF1_SEPARATOR  65535 // FFFFh
#define PSF1_STARTSEQ   65534 // FFFEh

type psf1_header 
  byte magic[2];     /* Magic number */
  byte mode;         /* PSF font mode */
  byte charsize;     /* Character size */
end;

//For psf2 this is

#define PSF2_MAGIC0     114 // 72h
#define PSF2_MAGIC1      91 // b5h
#define PSF2_MAGIC2      74 //4ah
#define PSF2_MAGIC3     134 //86h

/* bits used in flags */
#define PSF2_HAS_UNICODE_TABLE 1 //01h

/* max version recognized so far */
#define PSF2_MAXVERSION 0

/* UTF8 separators */
#define PSF2_SEPARATOR  255 //FFh
#define PSF2_STARTSEQ   254 //FEh

type psf2_header 
	byte magic[4];
	int version;
	int headersize;    /* offset of bitmaps in file */
	int flags;
	int length;        /* number of glyphs */
	int charsize;      /* number of bytes for each character */
	int height, width; /* max dimensions of glyphs */
	/* charsize = height * ((width + 7) / 8) */
end;

/*----------------------------------------------------------------------*/

function int cfnt_load_psf(CFNT_FONT *afont, string afilename )
local
  string tt;
  int fd;
  psf1_header hd1;
  psf2_header hd2;
  int psfver;
  int bpline;
  int i,j,b,c;
  word uni;
end
begin
  fd = fopen( afilename, "rb" );
  if( !fd ) say("Font not found: "+afilename); return -1; end

  for( i=0; i<MAX_CFNT_GLYPHS; i++ )
    afont.encoding[i] = 0;
    afont.glmap[i] = 0;
    afont.uni2glyph[c] = 0;
  end;


  fseek( fd, 0, SEEK_SET );
  fread( &hd2.magic[0], 1, fd );
  fread( &hd2.magic[1], 1, fd );
  fread( &hd2.magic[2], 1, fd );
  fread( &hd2.magic[3], 1, fd );

  fseek( fd, 0, SEEK_SET );
  fread( &hd1.magic[0], 1, fd );
  fread( &hd1.magic[1], 1, fd );
  fread( &hd1.mode, 1, fd );
  fread( &hd1.charsize, 1, fd );
  
  psfver = 0;
// Check if it is PSF1:
  if( hd1.magic[0] == PSF1_MAGIC0 && hd1.magic[1] == PSF1_MAGIC1 ) psfver=1; end
// Check if it is PSF2:
  if( hd2.magic[0] == PSF2_MAGIC0 && hd2.magic[1] == PSF2_MAGIC1 &&
      hd2.magic[2] == PSF2_MAGIC2 && hd2.magic[3] == PSF2_MAGIC3
    ) psfver=2; end

  if( !psfver ) say("Bad magic: "+hd2.magic[0]+" "+hd2.magic[1]+" "+hd2.magic[2]+" "+hd2.magic[3]); fclose (fd); return -1; end;

  say("PSF Version: "+psfver);

  if( psfver==1 )
    afont.width  = 8;
    bpline = 1; // bytes per line;
    afont.height = hd1.charsize;
    afont.length = hd1.mode & PSF1_MODE512 ? 512: 256;
    if( hd1.mode & PSF1_MODEHASTAB ) say("Font has table."); end;
    if( hd1.mode & PSF1_MODEHASSEQ ) say("Font has sequences."); end;
    fseek( fd, 5, SEEK_SET );
  end;

  if( psfver==2 )
    say ("Version 2 is not supported yet :(");
    fclose (fd);
    return -2;
  end;

  say( "Font: " + afont.width + "x" + afont.height );
  say( "Characters: " + afont.length );

// reading font data
  fseek(fd, -1, SEEK_CUR ); //?? why

  for( c = 0; c < afont.length; c++ )
    for( j = 0; j < afont.height; j++ )
      //for( i = 0; i< bpline; i++ ) 
	fread( &afont.glyph[ c ][ j ]/*[ i ]*/, 1, fd );
	afont.uni2glyph[c] = 0;
      //end;
    end;
  end;

// reading unicode map

  if( ! feof(fd) )
    c = 0;
    while(!feof(fd))
      // char, char, char --- group char+char group char+char endmark
      // we are ignoring compound chars and expect only one unicode match per one glyph
      // 
      loop
	fread( &uni, 2, fd );
	if(uni == PSF1_STARTSEQ || uni == PSF1_SEPARATOR ) break; end;
	if( feof(fd) )
	    say("Unexpected end of unicode table.");
	    fclose (fd);
	    return -3;
	end;
	if( afont.uni2glyph[c] && uni < afont.uni2glyph[c])
	  afont.uni2glyph[c] = uni;
	end;
	if( ! afont.uni2glyph[c] )
	  afont.uni2glyph[c] = uni;
	end;
      end;
      /// read until end of group
      while( uni != PSF1_SEPARATOR )
	fread( &uni, 2, fd );
      end;
     
     // map ascii  to unicode
      if ( 127 != ( afont.uni2glyph[c] & 127 ) ) 
	afont.encoding[c] = afont.uni2glyph[c]; 
      end;
      c++;
    end;
    afont.asciifont = 0;
  else
   afont.asciifont = 1;
  end;

  fclose (fd);
  
  cfnt_reindex ( afont );
  cfnt_prerender_maps( afont );
  
  return 0;
end

/*----------------------------------------------------------------------*/

function int cfnt_load_raw(CFNT_FONT *afont, string afilename, int hgt )
local
  string tt;
  int fd;
  int bpline;
  int i,j,b,c;
  word uni;
end
begin
  fd = fopen( afilename, "rb" );
  if( !fd ) say("Font not found: "+afilename); return -1; end
  
  for( i=0; i<MAX_CFNT_GLYPHS; i++ )
    afont.encoding[i] = 0;
    afont.glmap[i] = 0;
    afont.uni2glyph[c] = 0;
  end;

  say("RAW font");

  afont.width  = 8;
  bpline = 1; // bytes per line;
  afont.height = hgt;
  afont.length = 256;
  afont.asciifont = 1;
  
  say( "Font: " + afont.width + "x" + afont.height );
  say( "Characters: " + afont.length );

  for( c = 0; c < afont.length; c++ )
    for( j = 0; j < afont.height; j++ )
      //for( i = 0; i< bpline; i++ ) 
	fread( &afont.glyph[ c ][ j ]/*[ i ]*/, 1, fd );
	afont.uni2glyph[c] = 0;
      //end;
    end;
  end;

  fclose (fd);
  cfnt_prerender_maps( afont );
  
  return 0;
end
