#ifndef __COMMANDER_INC__
#define __COMMANDER_INC__
/*
    FwFNav (Bennu Windowed File Navigator) is a menu program for GP2x Wiz game console.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

import "mod_file"
import "mod_say"
import "mod_dir"

GLOBAL
  string commander_cmd = "/dev/shm/comm.cmd";
  string commander_runprefix = "/dev/shm/task-run-";
  string commander_resprefix  = "/dev/shm/task-res-";
  string commander_command  = 0;
  int    commander_task_pid = 1;
END

TYPE FILE_INFO
    string name = "";
    string ext = "";
    int fsize = 0;
    int f_dir;
    int f_hidden;
    int f_exe;
    int f_link;
    string linkto = "";
    string opts = "";
END

/////////////////////////////////////////////////////////////////////////

FUNCTION COMMANDER_INIT(string cmd_pipe)
BEGIN
  commander_cmd = cmd_pipe;
END;

////////////////////////////////////////////////////////////////////////

FUNCTION LS2FILEINFO(string path, string ls, FILE_INFO *info)
LOCAL
  int p, p1, i, l;
  string ch, ich, link;
END
BEGIN
  info.f_link = info.f_exe = info.f_hidden = info.f_dir = 0;
  ch = glob(path+ls);
  IF( ch != "" )
    info.name  = fileinfo.name;  
    info.f_dir = fileinfo.directory;
    info.fsize = fileinfo.size;
  ELSE
    info.name  = "ERROR";  
    info.f_dir = "ERROR";
    info.fsize  = 0;
  END;
  glob("");
  
  IF(info.f_dir)
    info.ext = "";
  ELSE
    
    info.ext = lcase( strrev( info.name ) );
    i = find( info.ext, '.', 0 );
    IF( i > 0 )
      info.ext = strrev( substr( info.ext, 0, i ) );
    ELSE
      info.ext = "";
    END;
    
    IF( info.ext=="gpu" || info.ext=="gpe" ) 
      info.f_exe = 1;
    END;
    
  END;  
  //IF( ich=="/" ) info.f_dir = 1; END;
  //IF( find(ch,'*')<0 ) info.f_exe = 1; END;
  
  /*
  p=0;
  FOR(i=10;i<l;i++)
    IF(ls[i]==' ' && ls[i-1]!=' ') 
      p++; 
      IF( p==4 ) p1=i; END; 
      IF( p==5 ) 
	info.size = atoi(substr(ls,p1,i-p1));
      END; 
      IF(p==8) info.name=substr(ls,i+1,l-i-1); BREAK; END; 
    END;
  END;
  */  
  
  ///info.name = UNESCAPELS(info.name, &info.linkto);
//  link = info.linkto;
/*  UNESCAPELS(&info.name, &info.linkto);
  IF(info.linkto != "")
    info.f_link = 1;
    say("link");
  END;*/
  
END;

////////////////////////////////////////////////////////////////////////

FUNCTION string ESCAPESHELLARG(string ls)
LOCAL
  string escape[]=("'", "");
  int i,j,l;
  string obuf;
  string tmp;
END
BEGIN
  tmp = ls;
  FOR( i=0; escape[i]!=""; i++)
    obuf = "";
    l = len(tmp);
    
    FOR(j=0;j<l;j++)
      IF( tmp[j]==escape[i] )
	obuf += "\"+tmp[j];
      ELSE
	obuf += tmp[j];
      END;
    END;
    tmp = obuf;

  END;
  RETURN "'" + tmp + "'";
END;

////////////////////////////////////////////////////////////////////////

// parsing ls -QF output
FUNCTION UNESCAPELS(string *argx, string *linkto)
LOCAL
  int i,l;
  string obuf, arg;
  string stop, lnk="";
END
BEGIN
  arg = *argx;
  obuf = "";
  l = len(arg);
  stop = arg[0];
  
  FOR(i=1; i<l-1; i++)
    IF(arg[i]==stop) BREAK; END;
    IF(arg[i]=='\') 
      i++;
      obuf += arg[i];
      CONTINUE; 
    END;
    obuf += arg[i];
  END;

  i+=2;
  /// it's a symlink
  IF(i<l)
    // pass " -> "
    FOR(; arg[i] != stop && i<l-1; i++) END;
    FOR(i++; i<l-1; i++)
      IF(arg[i]==stop) BREAK; END;
      IF(arg[i]=='\') 
	i++;
	lnk += arg[i];
	CONTINUE; 
      END;
      lnk += arg[i];
    END;
  END;
  
  *linkto = lnk;  
  *argx = obuf;
END;

GLOBAL
END

////////////////////////////////////////////////////////////////////////

FUNCTION int TASK_IS_RUNNING(int pid)
BEGIN
  return file_exists( commander_runprefix + pid );
END;

////////////////////////////////////////////////////////////////////////

FUNCTION string TASK_RES_FILE(int pid)
BEGIN
  return commander_resprefix + pid ;
END;
////////////////////////////////////////////////////////////////////////

FUNCTION int TASK_CLEANUP(int pid)
BEGIN
  // if ( file_exists( task_res_file(pid) ) ) rm( task_res_file(pid) ); end;
END;

////////////////////////////////////////////////////////////////////////

FUNCTION COMMANDER_TASK(string command, int sync)
LOCAL
  int fd , ft, pid, freeze;
  string xbuf;
END
BEGIN
  say( "COMMANDER_TASK(" + command + ")" );
  pid = commander_task_pid ++;
  
  fd = fopen(commander_cmd, O_WRITE);
  IF(!fd)
    RETURN -1;
  END;
  
  ft = fopen( commander_runprefix + pid , O_WRITE);
  IF( !ft )
    fclose( ft );
    RETURN -1;
  END;
  
  xbuf  = "TASK" + chr(10);
  xbuf += commander_runprefix + pid + chr(10);
  xbuf += commander_resprefix + pid + chr(10);
  xbuf += command + chr(10);
  
  say(xbuf);
  
  fputs(fd, xbuf);
  fclose(fd);
  
  IF(!sync)
    // bg task
    RETURN pid;
  ELSE
    // witing for results
    say("waiting");
    freeze = 100;
    WHILE ( task_is_running( pid ) && --freeze)
      FRAME;
    END;
    if( !freeze )
      say("Task is frozen! Reporting fail!");
      return -1; 
    end;
    say("task done");
    RETURN pid;
  END;
END;

////////////////////////////////////////////////////////////////////////


#endif