#ifndef __INUPT_MODULE_INC__
#define __INUPT_MODULE_INC__ 1
/*
    Bennu Convenience Lib Set.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

#ifndef USING_FENIX
import "mod_key";
import "mod_joy";
import "mod_proc";
import "mod_say";
import "mod_file";
import "mod_mem";
import "mod_string";
import "mod_timers";
#endif

/* -------------------------------------------------------------------------- */

global
    string im_key_name[]= (
    "UP",
    "UPLEFT",
    "LEFT",
    "DOWNLEFT",
    "DOWN",
    "DOWNRIGHT",
    "RIGHT",
    "UPRIGHT",
    "MENU/START",
    "SELECT",
    "L",
    "R",
    "A",
    "B",
    "X",
    "Y",
    "VOLUP",
    "VOLDOWN",
    "CLICK",
    ""
    );
    int im_key_configurable[]= (
    1, // "UP",
    0, // "UPLEFT",
    1, // "LEFT",
    0, // "DOWNLEFT",
    1, // "DOWN",
    0, // "DOWNRIGHT",
    1, // "RIGHT",
    0, // "UPRIGHT",
    1, // "MENU",
    1, // "SELECT",
    1, // "L",
    1, // "R",
    1, // "A",
    1, // "B",
    1, // "X",
    1, // "Y",
    1, // "VOLUP",
    1, // "VOLDOWN",
    0, // "CLICK",
    );
    
end

const
    _JKEY_UP        =  0 ;
    _JKEY_UPLEFT    =  1 ;
    _JKEY_LEFT      =  2 ;
    _JKEY_DOWNLEFT  =  3 ;
    _JKEY_DOWN      =  4 ;
    _JKEY_DOWNRIGHT =  5 ;
    _JKEY_RIGHT     =  6 ;
    _JKEY_UPRIGHT   =  7 ;
    _JKEY_MENU      =  8 ;
    _JKEY_SELECT    =  9 ;
    _JKEY_L         = 10 ;
    _JKEY_R         = 11 ;
    _JKEY_A         = 12 ;
    _JKEY_B         = 13 ;
    _JKEY_X         = 14 ;
    _JKEY_Y         = 15 ;
    _JKEY_VOLUP     = 16 ;
    _JKEY_VOLDOWN   = 17 ;
    _JKEY_CLICK     = 18 ;    
    _JKEY_LAST      = 19 ;
end

/* -------------------------------------------------------------------------- */

type input_device_profile
  string name = "Default KB";
  int get_kb  = 1; // read kb
  int get_joy = 0; // read joystick
  int jn   = 0; // which joystick
  int jbn  = 0; // number of joystick buttons
  int jkeys_kb [_JKEY_LAST]; // keyboard to metakeys bindings
  int jkeys_joy[_JKEY_LAST]; // joystick to metakeys bindings
  int jkeys_state[_JKEY_LAST]; // real input state
end;

#define INPUT_MAX_INPUTDEVS 10

/* -------------------------------------------------------------------------- */

// if code > _IVEV_JOY ... it is joy
const
  _IEV_JOY	  = 500;  
  _IEV_JOY_UP     = 501;
  _IEV_JOY_DOWN   = 502;
  _IEV_JOY_LEFT   = 503;
  _IEV_JOY_RIGHT  = 504;
  _IEV_JOY_BUTTON = 1000;
end;

/* -------------------------------------------------------------------------- */

global
  int im_jkey_grab    = 0;
  int im_jkey_allowed = 0;
  
  input_device_profile im_devs[ INPUT_MAX_INPUTDEVS ];
  int im_dev_count   = 0;
  input_device_profile *im_default_dev;
end

/* -------------------------------------------------------------------------- */
// tell an input device name
function string input_dev_name(int i)
begin
  if( i>=0 && i<im_dev_count ) return im_devs[i].name; end;
  return "";
end

/* -------------------------------------------------------------------------- */
// flush dev buffers
function input_init_dev(int i)
local int j; end
begin
  if( i>=0 && i<im_dev_count ) 
    for(j=0;j<_JKEY_LAST;j++)
      im_devs[i].jkeys_state[j]=0;
    end
  end;
end

/* -------------------------------------------------------------------------- */
// set a default input device
function input_set_dev(int i)
begin
  if( i>=0 && i<im_dev_count ) 
    say("Input set to: "+im_devs[i].name);
    im_default_dev = &(im_devs[i]);
  end;
end

/* -------------------------------------------------------------------------- */

// "Fair" clients should use jkey() just like system key()
// or specific jkey* functions
function jkey(int jkeymeta)
begin
  if( jkeymeta<0 && jkeymeta >= _JKEY_LAST ) return 0; end
    
  return im_default_dev.jkeys_state[jkeymeta];
end;

/* -------------------------------------------------------------------------- */
// direct input from input device idev
function jdkey(int jkeymeta, int idev)
begin
  if( idev>=0 && idev<im_dev_count ) 
    if( jkeymeta<0 && jkeymeta >= _JKEY_LAST ) return 0; end      
    return im_devs[idev].jkeys_state[jkeymeta];
  end
  return 0;
end;

/* -------------------------------------------------------------------------- */

// "allowed" jkey, all subprocess (window contents or so) shoudld use this
// myid - id of claller procees
function ajkey(int jkeymeta, int myid)
begin
  if( jkeymeta<0 && jkeymeta >= _JKEY_LAST ) return 0; end
    
  if( im_jkey_grab > 0 )
    if( myid == im_jkey_allowed || myid == im_jkey_grab )
      //if(im_default_dev.jkeys_state[jkeymeta]) say('rd filtered from '+myid); end
      return im_default_dev.jkeys_state[jkeymeta];
    else
      return 0;
    end
  else
    //if(im_default_dev.jkeys_state[jkeymeta]) say('rd FREE from '+myid); end
    return im_default_dev.jkeys_state[jkeymeta];
  end;
  
end;

/* -------------------------------------------------------------------------- */
function input_grab( int to_who )
begin
  //say ( "Input grabbed by "+to_who );
  im_jkey_allowed = 0;
  im_jkey_grab    = to_who;
end;

/* -------------------------------------------------------------------------- */
function input_release()
begin
  im_jkey_allowed = 0;
  im_jkey_grab    = 0;
end;

/* -------------------------------------------------------------------------- */
function input_allow(int to_who)
begin
  //if( im_jkey_allowed != to_who) say ( "Input allowed to "+to_who ); end;
  im_jkey_allowed = to_who;
end;

/* -------------------------------------------------------------------------- */

function input_is_sane()
local
  int lst[7], i, j, kbok, joyok;
end
begin
  kbok  = 1;
  joyok = 1;
  // arrows, a,b, menu must be set, 
  // and set to dirrefent keys
  // kb
  if( ! (lst[ 0 ] = im_default_dev.jkeys_kb[ _JKEY_UP ]) )  kbok = 0; end
  if( ! (lst[ 1 ] = im_default_dev.jkeys_kb[ _JKEY_DOWN ]) ) kbok = 0; end
  if( ! (lst[ 2 ] = im_default_dev.jkeys_kb[ _JKEY_LEFT ]) ) kbok = 0; end
  if( ! (lst[ 3 ] = im_default_dev.jkeys_kb[ _JKEY_RIGHT ]) ) kbok = 0; end
  if( ! (lst[ 5 ] = im_default_dev.jkeys_kb[ _JKEY_A ]) ) kbok = 0; end
  if( ! (lst[ 5 ] = im_default_dev.jkeys_kb[ _JKEY_B ]) ) kbok = 0; end
  if( ! (lst[ 6 ] = im_default_dev.jkeys_kb[ _JKEY_MENU ]) ) kbok = 0; end
  for(i = 0; i < 7; i++ )
    for(j = i+1; j < 7; j++ )
      if( lst[ i ]==lst[ j ] ) kbok = 0; end
    end
  end
  
  // joy
  if( ! (lst[ 0 ] = im_default_dev.jkeys_joy[ _JKEY_UP ]) ) joyok = 0; end
  if( ! (lst[ 1 ] = im_default_dev.jkeys_joy[ _JKEY_DOWN ]) ) joyok = 0; end
  if( ! (lst[ 2 ] = im_default_dev.jkeys_joy[ _JKEY_LEFT ]) ) joyok = 0; end
  if( ! (lst[ 3 ] = im_default_dev.jkeys_joy[ _JKEY_RIGHT ]) ) joyok = 0; end
  if( ! (lst[ 5 ] = im_default_dev.jkeys_joy[ _JKEY_A ]) ) joyok = 0; end
  if( ! (lst[ 5 ] = im_default_dev.jkeys_joy[ _JKEY_B ]) ) joyok = 0; end
  if( ! (lst[ 6 ] = im_default_dev.jkeys_joy[ _JKEY_MENU ]) ) joyok = 0; end
  for(i = 0; i < 7; i++ )
    for(j = i+1; j < 7; j++ )
      if( lst[ i ]==lst[ j ] ) joyok = 0; end
    end
  end
  
  return joyok || kbok;
end

/* -------------------------------------------------------------------------- */

function input_preset_safe()
begin
    say("Safe Preset. "+im_default_dev.name);
    im_default_dev.jkeys_kb[ _JKEY_UP        ] = _UP ;
    im_default_dev.jkeys_kb[ _JKEY_UPLEFT    ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_LEFT      ] = _LEFT ;
    im_default_dev.jkeys_kb[ _JKEY_DOWNLEFT  ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_DOWN      ] = _DOWN ;
    im_default_dev.jkeys_kb[ _JKEY_DOWNRIGHT ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_RIGHT     ] = _RIGHT ;
    im_default_dev.jkeys_kb[ _JKEY_UPRIGHT   ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_MENU      ] = _ESC ;
    im_default_dev.jkeys_kb[ _JKEY_SELECT    ] = _SPACE ;
    im_default_dev.jkeys_kb[ _JKEY_L         ] = _PGUP ;
    im_default_dev.jkeys_kb[ _JKEY_R         ] = _PGDN ;
    im_default_dev.jkeys_kb[ _JKEY_A         ] = _CONTROL ;
    im_default_dev.jkeys_kb[ _JKEY_B         ] = _ENTER ;
    im_default_dev.jkeys_kb[ _JKEY_X         ] = _ALT ;
    im_default_dev.jkeys_kb[ _JKEY_Y         ] = _L_SHIFT ;
    im_default_dev.jkeys_kb[ _JKEY_VOLUP     ] = _PLUS ;
    im_default_dev.jkeys_kb[ _JKEY_VOLDOWN   ] = _MINUS ;
    im_default_dev.jkeys_kb[ _JKEY_CLICK     ] = 0 ;
    im_default_dev.jkeys_joy[ _JKEY_UP        ] = _IEV_JOY_UP ;
    im_default_dev.jkeys_joy[ _JKEY_UPLEFT    ] = 0 ;
    im_default_dev.jkeys_joy[ _JKEY_LEFT      ] = _IEV_JOY_LEFT ;
    im_default_dev.jkeys_joy[ _JKEY_DOWNLEFT  ] = 0 ;
    im_default_dev.jkeys_joy[ _JKEY_DOWN      ] = _IEV_JOY_DOWN ;
    im_default_dev.jkeys_joy[ _JKEY_DOWNRIGHT ] = 0 ;
    im_default_dev.jkeys_joy[ _JKEY_RIGHT     ] = _IEV_JOY_RIGHT ;
    im_default_dev.jkeys_joy[ _JKEY_UPRIGHT   ] = 0 ;
    im_default_dev.jkeys_joy[ _JKEY_MENU      ] = _IEV_JOY_BUTTON +  2;
    im_default_dev.jkeys_joy[ _JKEY_SELECT    ] = _IEV_JOY_BUTTON +  9;
    im_default_dev.jkeys_joy[ _JKEY_L         ] = _IEV_JOY_BUTTON +  7;
    im_default_dev.jkeys_joy[ _JKEY_R         ] = _IEV_JOY_BUTTON +  8;
    im_default_dev.jkeys_joy[ _JKEY_A         ] = _IEV_JOY_BUTTON +  0 ;
    im_default_dev.jkeys_joy[ _JKEY_B         ] = _IEV_JOY_BUTTON +  1 ;
    im_default_dev.jkeys_joy[ _JKEY_X         ] = _IEV_JOY_BUTTON +  3;
    im_default_dev.jkeys_joy[ _JKEY_Y         ] = _IEV_JOY_BUTTON +  4;
    im_default_dev.jkeys_joy[ _JKEY_VOLUP     ] = _IEV_JOY_BUTTON +  5;
    im_default_dev.jkeys_joy[ _JKEY_VOLDOWN   ] = _IEV_JOY_BUTTON +  6;
    im_default_dev.jkeys_joy[ _JKEY_CLICK     ] = 0;
end

function input_preset_pc()
begin
    say("Keyboard init for PC classic "+im_default_dev.name);
    im_default_dev.jkeys_kb[ _JKEY_UP        ] = _UP ;
    im_default_dev.jkeys_kb[ _JKEY_UPLEFT    ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_LEFT      ] = _LEFT ;
    im_default_dev.jkeys_kb[ _JKEY_DOWNLEFT  ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_DOWN      ] = _DOWN ;
    im_default_dev.jkeys_kb[ _JKEY_DOWNRIGHT ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_RIGHT     ] = _RIGHT ;
    im_default_dev.jkeys_kb[ _JKEY_UPRIGHT   ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_MENU      ] = _ESC ;
    im_default_dev.jkeys_kb[ _JKEY_SELECT    ] = _SPACE ;
    im_default_dev.jkeys_kb[ _JKEY_L         ] = _PGUP ;
    im_default_dev.jkeys_kb[ _JKEY_R         ] = _PGDN ;
    im_default_dev.jkeys_kb[ _JKEY_A         ] = _CONTROL ;
    im_default_dev.jkeys_kb[ _JKEY_B         ] = _ENTER ;
    im_default_dev.jkeys_kb[ _JKEY_X         ] = _ALT ;
    im_default_dev.jkeys_kb[ _JKEY_Y         ] = _L_SHIFT ;
    im_default_dev.jkeys_kb[ _JKEY_VOLUP     ] = _PLUS ;
    im_default_dev.jkeys_kb[ _JKEY_VOLDOWN   ] = _MINUS ;
    im_default_dev.jkeys_kb[ _JKEY_CLICK     ] = 0 ;
end;

function input_preset_pc2()
begin
    say("Keyboard init for PC as GPH. "+im_default_dev.name);
    im_default_dev.jkeys_kb[ _JKEY_UP        ] = _W ;
    im_default_dev.jkeys_kb[ _JKEY_UPLEFT    ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_LEFT      ] = _A ;
    im_default_dev.jkeys_kb[ _JKEY_DOWNLEFT  ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_DOWN      ] = _S ;
    im_default_dev.jkeys_kb[ _JKEY_DOWNRIGHT ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_RIGHT     ] = _D ;
    im_default_dev.jkeys_kb[ _JKEY_UPRIGHT   ] = 0 ;
    im_default_dev.jkeys_kb[ _JKEY_MENU      ] = _ESC ;
    im_default_dev.jkeys_kb[ _JKEY_SELECT    ] = _ENTER ;
    im_default_dev.jkeys_kb[ _JKEY_L         ] = _Q ;
    im_default_dev.jkeys_kb[ _JKEY_R         ] = _E ;
    im_default_dev.jkeys_kb[ _JKEY_A         ] = _F ;
    im_default_dev.jkeys_kb[ _JKEY_B         ] = _H ;
    im_default_dev.jkeys_kb[ _JKEY_X         ] = _G ;
    im_default_dev.jkeys_kb[ _JKEY_Y         ] = _J ;
    im_default_dev.jkeys_kb[ _JKEY_VOLUP     ] = _O ;
    im_default_dev.jkeys_kb[ _JKEY_VOLDOWN   ] = _L ;
    im_default_dev.jkeys_kb[ _JKEY_CLICK     ] = 0 ;
end;

function input_preset_gph()
begin
    say("Joystick init for GPH. "+im_default_dev.name);
    im_default_dev.jkeys_joy[ _JKEY_UP        ] = _IEV_JOY_BUTTON + _JKEY_UP ;
    im_default_dev.jkeys_joy[ _JKEY_UPLEFT    ] = _IEV_JOY_BUTTON + _JKEY_UPLEFT ;
    im_default_dev.jkeys_joy[ _JKEY_LEFT      ] = _IEV_JOY_BUTTON +  _JKEY_LEFT ;
    im_default_dev.jkeys_joy[ _JKEY_DOWNLEFT  ] = _IEV_JOY_BUTTON +  _JKEY_DOWNLEFT ;
    im_default_dev.jkeys_joy[ _JKEY_DOWN      ] = _IEV_JOY_BUTTON +  _JKEY_DOWN ;
    im_default_dev.jkeys_joy[ _JKEY_DOWNRIGHT ] = _IEV_JOY_BUTTON +  _JKEY_DOWNRIGHT ;
    im_default_dev.jkeys_joy[ _JKEY_RIGHT     ] = _IEV_JOY_BUTTON +  _JKEY_RIGHT ;
    im_default_dev.jkeys_joy[ _JKEY_UPRIGHT   ] = _IEV_JOY_BUTTON +  _JKEY_UPRIGHT ;
    im_default_dev.jkeys_joy[ _JKEY_MENU      ] = _IEV_JOY_BUTTON +  _JKEY_MENU ;
    im_default_dev.jkeys_joy[ _JKEY_SELECT    ] = _IEV_JOY_BUTTON +  _JKEY_SELECT ;
    im_default_dev.jkeys_joy[ _JKEY_L         ] = _IEV_JOY_BUTTON +  _JKEY_L ;
    im_default_dev.jkeys_joy[ _JKEY_R         ] = _IEV_JOY_BUTTON +  _JKEY_R ;
    im_default_dev.jkeys_joy[ _JKEY_A         ] = _IEV_JOY_BUTTON +  _JKEY_A ;
    im_default_dev.jkeys_joy[ _JKEY_B         ] = _IEV_JOY_BUTTON +  _JKEY_B ;
    im_default_dev.jkeys_joy[ _JKEY_X         ] = _IEV_JOY_BUTTON +  _JKEY_X ;
    im_default_dev.jkeys_joy[ _JKEY_Y         ] = _IEV_JOY_BUTTON +  _JKEY_Y ;
    im_default_dev.jkeys_joy[ _JKEY_VOLUP     ] = _IEV_JOY_BUTTON +  _JKEY_VOLUP ;
    im_default_dev.jkeys_joy[ _JKEY_VOLDOWN   ] = _IEV_JOY_BUTTON +  _JKEY_VOLDOWN ;
    im_default_dev.jkeys_joy[ _JKEY_CLICK     ] = _IEV_JOY_BUTTON +  _JKEY_CLICK ;
end;

function input_load_config(string fname)
local
  string pnames[]= (
  "KB_JKEY_UP",
  "KB_JKEY_UPLEFT",
  "KB_JKEY_LEFT",
  "KB_JKEY_DOWNLEFT",
  "KB_JKEY_DOWN",
  "KB_JKEY_DOWNRIGHT",
  "KB_JKEY_RIGHT",
  "KB_JKEY_UPRIGHT",
  "KB_JKEY_MENU",
  "KB_JKEY_SELECT",
  "KB_JKEY_L",
  "KB_JKEY_R",
  "KB_JKEY_A",
  "KB_JKEY_B",
  "KB_JKEY_X",
  "KB_JKEY_Y",
  "KB_JKEY_VOLUP",
  "KB_JKEY_VOLDOWN",
  "KB_JKEY_CLICK",
  "JOY_JKEY_UP",
  "JOY_JKEY_UPLEFT",
  "JOY_JKEY_LEFT",
  "JOY_JKEY_DOWNLEFT",
  "JOY_JKEY_DOWN",
  "JOY_JKEY_DOWNRIGHT",
  "JOY_JKEY_RIGHT",
  "JOY_JKEY_UPRIGHT",
  "JOY_JKEY_MENU",
  "JOY_JKEY_SELECT",
  "JOY_JKEY_L",
  "JOY_JKEY_R",
  "JOY_JKEY_A",
  "JOY_JKEY_B",
  "JOY_JKEY_X",
  "JOY_JKEY_Y",
  "JOY_JKEY_VOLUP",
  "JOY_JKEY_VOLDOWN",
  "JOY_JKEY_CLICK",
  ""
  );
  int ptypes[]= (
  0,//1
  0,
  0,
  0,
  0,//5
  0,
  0,
  0,
  0,
  0,//10
  0,
  0,
  0,
  0,
  0,//15
  0,
  0,
  0,
  0,//19
  1,//1
  1,
  1,
  1,
  1,//5
  1,
  1,
  1,
  1,
  1,//10
  1,
  1,
  1,
  1,
  1,//15
  1,
  1,
  1,
  1,//19
  );
  int pcodes[]= (
  _JKEY_UP,
  _JKEY_UPLEFT,
  _JKEY_LEFT,
  _JKEY_DOWNLEFT,
  _JKEY_DOWN,
  _JKEY_DOWNRIGHT,
  _JKEY_RIGHT,
  _JKEY_UPRIGHT,
  _JKEY_MENU,
  _JKEY_SELECT,
  _JKEY_L,
  _JKEY_R,
  _JKEY_A,
  _JKEY_B,
  _JKEY_X,
  _JKEY_Y,
  _JKEY_VOLUP,
  _JKEY_VOLDOWN,
  _JKEY_CLICK,
  _JKEY_UP,
  _JKEY_UPLEFT,
  _JKEY_LEFT,
  _JKEY_DOWNLEFT,
  _JKEY_DOWN,
  _JKEY_DOWNRIGHT,
  _JKEY_RIGHT,
  _JKEY_UPRIGHT,
  _JKEY_MENU,
  _JKEY_SELECT,
  _JKEY_L,
  _JKEY_R,
  _JKEY_A,
  _JKEY_B,
  _JKEY_X,
  _JKEY_Y,
  _JKEY_VOLUP,
  _JKEY_VOLDOWN,
  _JKEY_CLICK
  );
  int i,j,n,fd,*keys;
  string pn,pv,buf;
end
begin
    say("Loading input config from "+fname);
    fd = fopen(fname, O_READ);
    if(!fd)
      say("Cannot open file");
      return;
    end;

    for( n=0; pnames[n]!=""; n++ ) end;
    keys = alloc( sizeof(int)*n );
    for( i=0; i<n; i++ ) keys[i] = 0; end;
    
    while( !feof ( fd ) )
      buf = fgets(fd);
      j = find(buf,"=");
      if( j>0 )
	pn = substr(buf,0,j);
	pv = substr(buf,j+1,len(buf)-j-1);
	//say(pn+"="+pv);
	for ( i=0; i<n; i++ )
	  if ( pnames[i]==pn ) keys[i] = atoi(pv); break; end;
	end;
      end;
    end;
    
    for( i=0; i<n; i++ )
      if( ptypes[i]==0 )
	im_default_dev.jkeys_kb [ pcodes[i] ] = keys[i];
      else
	im_default_dev.jkeys_joy[ pcodes[i] ] = keys[i];
      end;
    end;

    free(keys); 
    fclose(fd);
end;

function input_save_config(string fname)
local
  int fd;
end
begin
    say("Saving input config to "+fname);
    fd = fopen(fname, O_WRITE);
    if(!fd)
      say("Cannot open file");
      return;
    end;
    fputs(fd, "KB_JKEY_UP=" +        im_default_dev.jkeys_kb[ _JKEY_UP        ] );
    fputs(fd, "KB_JKEY_UPLEFT=" +    im_default_dev.jkeys_kb[ _JKEY_UPLEFT    ] );
    fputs(fd, "KB_JKEY_LEFT=" +      im_default_dev.jkeys_kb[ _JKEY_LEFT      ] );
    fputs(fd, "KB_JKEY_DOWNLEFT=" +  im_default_dev.jkeys_kb[ _JKEY_DOWNLEFT  ] );
    fputs(fd, "KB_JKEY_DOWN=" +      im_default_dev.jkeys_kb[ _JKEY_DOWN      ] );
    fputs(fd, "KB_JKEY_DOWNRIGHT=" + im_default_dev.jkeys_kb[ _JKEY_DOWNRIGHT ] );
    fputs(fd, "KB_JKEY_RIGHT=" +     im_default_dev.jkeys_kb[ _JKEY_RIGHT     ] );
    fputs(fd, "KB_JKEY_UPRIGHT=" +   im_default_dev.jkeys_kb[ _JKEY_UPRIGHT   ] );
    fputs(fd, "KB_JKEY_MENU=" +      im_default_dev.jkeys_kb[ _JKEY_MENU      ] );
    fputs(fd, "KB_JKEY_SELECT=" +    im_default_dev.jkeys_kb[ _JKEY_SELECT    ] );
    fputs(fd, "KB_JKEY_L=" + im_default_dev.jkeys_kb[ _JKEY_L ] );
    fputs(fd, "KB_JKEY_R=" + im_default_dev.jkeys_kb[ _JKEY_R ] );
    fputs(fd, "KB_JKEY_A=" + im_default_dev.jkeys_kb[ _JKEY_A ] );
    fputs(fd, "KB_JKEY_B=" + im_default_dev.jkeys_kb[ _JKEY_B ] );
    fputs(fd, "KB_JKEY_X=" + im_default_dev.jkeys_kb[ _JKEY_X ] );
    fputs(fd, "KB_JKEY_Y=" + im_default_dev.jkeys_kb[ _JKEY_Y ] );
    fputs(fd, "KB_JKEY_VOLUP=" +   im_default_dev.jkeys_kb[ _JKEY_VOLUP   ] );
    fputs(fd, "KB_JKEY_VOLDOWN=" + im_default_dev.jkeys_kb[ _JKEY_VOLDOWN ] );
    fputs(fd, "KB_JKEY_CLICK=" +   im_default_dev.jkeys_kb[ _JKEY_CLICK   ] );
    
    fputs(fd, "JOY_JKEY_UP=" +        im_default_dev.jkeys_joy[ _JKEY_UP        ] );
    fputs(fd, "JOY_JKEY_UPLEFT=" +    im_default_dev.jkeys_joy[ _JKEY_UPLEFT    ] );
    fputs(fd, "JOY_JKEY_LEFT=" +      im_default_dev.jkeys_joy[ _JKEY_LEFT      ] );
    fputs(fd, "JOY_JKEY_DOWNLEFT=" +  im_default_dev.jkeys_joy[ _JKEY_DOWNLEFT  ] );
    fputs(fd, "JOY_JKEY_DOWN=" +      im_default_dev.jkeys_joy[ _JKEY_DOWN      ] );
    fputs(fd, "JOY_JKEY_DOWNRIGHT=" + im_default_dev.jkeys_joy[ _JKEY_DOWNRIGHT ] );
    fputs(fd, "JOY_JKEY_RIGHT=" +     im_default_dev.jkeys_joy[ _JKEY_RIGHT     ] );
    fputs(fd, "JOY_JKEY_UPRIGHT=" +   im_default_dev.jkeys_joy[ _JKEY_UPRIGHT   ] );
    fputs(fd, "JOY_JKEY_MENU=" +      im_default_dev.jkeys_joy[ _JKEY_MENU      ] );
    fputs(fd, "JOY_JKEY_SELECT=" +    im_default_dev.jkeys_joy[ _JKEY_SELECT    ] );
    fputs(fd, "JOY_JKEY_L=" + im_default_dev.jkeys_joy[ _JKEY_L ] );
    fputs(fd, "JOY_JKEY_R=" + im_default_dev.jkeys_joy[ _JKEY_R ] );
    fputs(fd, "JOY_JKEY_A=" + im_default_dev.jkeys_joy[ _JKEY_A ] );
    fputs(fd, "JOY_JKEY_B=" + im_default_dev.jkeys_joy[ _JKEY_B ] );
    fputs(fd, "JOY_JKEY_X=" + im_default_dev.jkeys_joy[ _JKEY_X ] );
    fputs(fd, "JOY_JKEY_Y=" + im_default_dev.jkeys_joy[ _JKEY_Y ] );
    fputs(fd, "JOY_JKEY_VOLUP=" +   im_default_dev.jkeys_joy[ _JKEY_VOLUP   ] );
    fputs(fd, "JOY_JKEY_VOLDOWN=" + im_default_dev.jkeys_joy[ _JKEY_VOLDOWN ] );
    fputs(fd, "JOY_JKEY_CLICK=" +   im_default_dev.jkeys_joy[ _JKEY_CLICK   ] );
    
    fclose(fd);
end;

function __input_dev_dump(int dev)
local
  input_device_profile *_dev;
end
begin
    say("Bindings of "+input_dev_name(dev) );
    _dev = &(im_devs[dev]);
    say("KB_JKEY_UP=" +        _dev.jkeys_kb[ _JKEY_UP        ] );
    say("KB_JKEY_UPLEFT=" +    _dev.jkeys_kb[ _JKEY_UPLEFT    ] );
    say("KB_JKEY_LEFT=" +      _dev.jkeys_kb[ _JKEY_LEFT      ] );
    say("KB_JKEY_DOWNLEFT=" +  _dev.jkeys_kb[ _JKEY_DOWNLEFT  ] );
    say("KB_JKEY_DOWN=" +      _dev.jkeys_kb[ _JKEY_DOWN      ] );
    say("KB_JKEY_DOWNRIGHT=" + _dev.jkeys_kb[ _JKEY_DOWNRIGHT ] );
    say("KB_JKEY_RIGHT=" +     _dev.jkeys_kb[ _JKEY_RIGHT     ] );
    say("KB_JKEY_UPRIGHT=" +   _dev.jkeys_kb[ _JKEY_UPRIGHT   ] );
    say("KB_JKEY_MENU=" +      _dev.jkeys_kb[ _JKEY_MENU      ] );
    say("KB_JKEY_SELECT=" +    _dev.jkeys_kb[ _JKEY_SELECT    ] );
    say("KB_JKEY_L=" + _dev.jkeys_kb[ _JKEY_L ] );
    say("KB_JKEY_R=" + _dev.jkeys_kb[ _JKEY_R ] );
    say("KB_JKEY_A=" + _dev.jkeys_kb[ _JKEY_A ] );
    say("KB_JKEY_B=" + _dev.jkeys_kb[ _JKEY_B ] );
    say("KB_JKEY_X=" + _dev.jkeys_kb[ _JKEY_X ] );
    say("KB_JKEY_Y=" + _dev.jkeys_kb[ _JKEY_Y ] );
    say("KB_JKEY_VOLUP=" +   _dev.jkeys_kb[ _JKEY_VOLUP   ] );
    say("KB_JKEY_VOLDOWN=" + _dev.jkeys_kb[ _JKEY_VOLDOWN ] );
    say("KB_JKEY_CLICK=" +   _dev.jkeys_kb[ _JKEY_CLICK   ] );
    
    say("JOY_JKEY_UP=" +        _dev.jkeys_joy[ _JKEY_UP        ] );
    say("JOY_JKEY_UPLEFT=" +    _dev.jkeys_joy[ _JKEY_UPLEFT    ] );
    say("JOY_JKEY_LEFT=" +      _dev.jkeys_joy[ _JKEY_LEFT      ] );
    say("JOY_JKEY_DOWNLEFT=" +  _dev.jkeys_joy[ _JKEY_DOWNLEFT  ] );
    say("JOY_JKEY_DOWN=" +      _dev.jkeys_joy[ _JKEY_DOWN      ] );
    say("JOY_JKEY_DOWNRIGHT=" + _dev.jkeys_joy[ _JKEY_DOWNRIGHT ] );
    say("JOY_JKEY_RIGHT=" +     _dev.jkeys_joy[ _JKEY_RIGHT     ] );
    say("JOY_JKEY_UPRIGHT=" +   _dev.jkeys_joy[ _JKEY_UPRIGHT   ] );
    say("JOY_JKEY_MENU=" +      _dev.jkeys_joy[ _JKEY_MENU      ] );
    say("JOY_JKEY_SELECT=" +    _dev.jkeys_joy[ _JKEY_SELECT    ] );
    say("JOY_JKEY_L=" + _dev.jkeys_joy[ _JKEY_L ] );
    say("JOY_JKEY_R=" + _dev.jkeys_joy[ _JKEY_R ] );
    say("JOY_JKEY_A=" + _dev.jkeys_joy[ _JKEY_A ] );
    say("JOY_JKEY_B=" + _dev.jkeys_joy[ _JKEY_B ] );
    say("JOY_JKEY_X=" + _dev.jkeys_joy[ _JKEY_X ] );
    say("JOY_JKEY_Y=" + _dev.jkeys_joy[ _JKEY_Y ] );
    say("JOY_JKEY_VOLUP=" +   _dev.jkeys_joy[ _JKEY_VOLUP   ] );
    say("JOY_JKEY_VOLDOWN=" + _dev.jkeys_joy[ _JKEY_VOLDOWN ] );
    say("JOY_JKEY_CLICK=" +   _dev.jkeys_joy[ _JKEY_CLICK   ] );

end;

/* -------------------------------------------------------------------------- */

function __im_assign_jkey2event ( int ivdev, int jkey_code, int iev_code )
local
  input_device_profile *d;
end
begin
  if( ivdev < 0 || ivdev >=  im_dev_count || jkey_code <0 || jkey_code >= _JKEY_LAST ) return; end
  d = &( im_devs[ ivdev ] );
  if( iev_code < _IEV_JOY )
    d.jkeys_kb[ jkey_code ]  = iev_code;
  else
    d.jkeys_joy[ jkey_code ] = iev_code;
  end
end

/* -------------------------------------------------------------------------- */

// Нужно ОБЯЗАТЕЛЬНО поддерживать __im_get_iev и input_module 
// в актуальном состоянии совместно !
function int __im_get_iev( int ivdev )
local
  int i, ax, ay; 
  //int joyid, int numkeys, 
  int thres;
  input_device_profile *d;
end
begin
  if( ivdev < 0 || ivdev >=  im_dev_count ) return; end
  d = &( im_devs[ ivdev ] );
  
  if(d.get_kb)
    // kb 
    // kb scancodes are numbers 1..99
    for ( i=1; i<100; i++)
    // BUG: lock status is read under wine, not press / release, so ignore them by  now
      if( key(i) && i != _CAPS_LOCK && i != _NUM_LOCK && i != _SCROLL_LOCK) return i; end
    end;
  end
  
  if(d.get_joy)
    thres = 10000;
    ax = joy_getaxis(d.jn, 0);
    ay = joy_getaxis(d.jn, 1);
	
    if ( ax >  thres )
      return _IEV_JOY_RIGHT;
    end;

    if ( ax < -thres )
      return _IEV_JOY_LEFT;
    end;

    if ( ay >  thres )
      return _IEV_JOY_DOWN;
    end;

    if ( ay < -thres )
      return _IEV_JOY_UP;
    end;

    for ( i=0; i<d.jbn; i++ )
      if ( joy_getbutton( d.jn, i ) )
	return _IEV_JOY_BUTTON + i;
      end;
    end;
  end;
  return 0;
end

/* -------------------------------------------------------------------------- */

process input_module(int refresh)
private
    int k, i , j, ax, ay;
    int jn, maxjbn; // selected joystick
    int thres = 10000;
    int *jkb;
    string t;
end
begin
    say("Input mopdule start...");
    //set_input_pc();
    
    i = 0;
    im_devs[i].name = "Keyboard";
    im_devs[i].get_kb  = 1;
    im_devs[i].get_joy = 0;
    
    jn = joy_number();
    
    //bug
    if( os_id==9  && jn==0 ) jn=1; end;
      
    say('JN='+jn);
    maxjbn = 0;
    for( i=0; (i < jn) && (i+1 < INPUT_MAX_INPUTDEVS); i++)
      im_devs[i+1].name = "Joystick "+(i+1);
      im_devs[i+1].get_kb  = 0;
      im_devs[i+1].get_joy = 1;
      im_devs[i+1].jn   = i;
      im_devs[i+1].jbn  = joy_buttons(i);
      if( im_devs[i+1].jbn > maxjbn ) maxjbn = im_devs[i+1].jbn; end
      say("Joystick found: "+im_devs[i+1].name+
      " ( "+im_devs[i+1].jbn+" buttons )");
    end
    im_dev_count = i+1;
    im_default_dev = &(im_devs[0]);
    
    // _IEV_JOY_RIGHT + _IEV_JOY_RIGHT + _IEV_JOY_DOWN + _IEV_JOY_UP + nbuttons
    // \--------------------------+-------------------------------/
    //                            v
    jkb = alloc ( sizeof(int) * ( 4 + maxjbn ) );
    for( k=0; k<im_dev_count; k++ )
      input_init_dev( k );
    end
    
    say("Input loop");
    loop
	frame refresh;
	for( k=0; k<im_dev_count; k++ )
	  if( im_devs[ k ].get_kb )
	    // KB loop
	    for ( i = 0; i < _JKEY_LAST; i++ )
		if ( im_devs[ k ].jkeys_kb[i] )
		    im_devs[ k ].jkeys_state[i] = key( im_devs[ k ].jkeys_kb[i] );
		else
		    im_devs[ k ].jkeys_state[i] = 0;
		end
	    end
	  else
	    for ( i = 0; i < _JKEY_LAST; i++ ) im_devs[ k ].jkeys_state[i] = 0; end;
	  end
	  if( im_devs[ k ].get_joy )
	    // Joy loop
	    if ( im_devs[ k ].jn>=0 )
		i = 0;
		ax = joy_getaxis(im_devs[ k ].jn, 0);
		ay = joy_getaxis(im_devs[ k ].jn, 1);
		    
		if ( ax >  thres )
		  jkb[i++] = _IEV_JOY_RIGHT;
		end;
	      
		if ( ax < -thres )
		  jkb[i++] = _IEV_JOY_LEFT;
		end;
	      
		if ( ay >  thres )
		  jkb[i++] = _IEV_JOY_DOWN;
		end;
	      
		if ( ay < -thres )
		  jkb[i++] = _IEV_JOY_UP;
		end;
		
		for ( j=0; j<im_devs[ k ].jbn; j++ )
		  if ( joy_getbutton( im_devs[ k ].jn, j ) )
		    jkb[i++] = _IEV_JOY_BUTTON + j;
		  end;
		end;
	      
		for ( i--; i >= 0; i-- )
		  for ( j =0; j < _JKEY_LAST; j++ )
		    if ( im_devs[ k ].jkeys_joy[j] == jkb[i] ) im_devs[ k ].jkeys_state[ j ] = 1; end;
		  end;
		end;
	    end;
	     /// old caanoo feature
	    if( os_id==9)
                    ax = joy_getaxis( 2, 0 );
                    ay = joy_getaxis( 2, 1 );

                    if ( ay < -16384 ) jkb[i++] = _IEV_JOY_UP; end;
                    if ( ax < -16384 ) jkb[i++] = _IEV_JOY_LEFT; end;
                    if ( ay >  16384 ) jkb[i++] = _IEV_JOY_DOWN; end;
                    if ( ax >  16384 ) jkb[i++] = _IEV_JOY_RIGHT; end;
	    end
	    /// GPH diagonal effect
	    if ( im_devs[ k ].jkeys_state[_JKEY_UPLEFT] )    im_devs[ k ].jkeys_state[_JKEY_UP]=1;   im_devs[ k ].jkeys_state[_JKEY_LEFT]=1;  end;
	    if ( im_devs[ k ].jkeys_state[_JKEY_UPRIGHT] )   im_devs[ k ].jkeys_state[_JKEY_UP]=1;   im_devs[ k ].jkeys_state[_JKEY_RIGHT]=1; end;
	    if ( im_devs[ k ].jkeys_state[_JKEY_DOWNLEFT] )  im_devs[ k ].jkeys_state[_JKEY_DOWN]=1; im_devs[ k ].jkeys_state[_JKEY_LEFT]=1;  end;
	    if ( im_devs[ k ].jkeys_state[_JKEY_DOWNRIGHT] ) im_devs[ k ].jkeys_state[_JKEY_DOWN]=1; im_devs[ k ].jkeys_state[_JKEY_RIGHT]=1; end;
	  end
	end
    end
    
    free(jkb);
end;

/* -------------------------------------------------------------------------- */
#endif