#ifndef USING_FENIX
import "mod_file";
import "mod_say";
import "mod_mem";
import "mod_rand";
import "mod_string";
#endif

/// a place to create temporary list files.
/// MUST BE mounted to memory! 
global
  list_tmp_prefix = "/dev/shm/lc";
end

type stringlist
  int ready = 0;
  string filename;
  int fd;
  int count;
  int allocated;
  int *idx;
  int readonly;
  
  string __last;
  int	 __last_idx;
end;

/*--------------------------------------------------------------------------*/

function string list_tmpfile_randomizer(string prefix)
local
  string res;
end
begin
  loop
    res = prefix+'-list-'+atoi( rand(1000,9999) );
    if( !file_exists(res) ) break; end;
  end;
  return res;
end;

/*--------------------------------------------------------------------------*/
/**
* Create stringlist from file.
* the idea is that file located in memory for example
* in /dev/shm
*/
function list_create_fn( stringlist *l, string filename )
local
  int cn;
end
begin
  l.ready = 0;
  l.readonly = 1;
  l.fd = fopen( filename, O_READ );
  if( l.fd )
    cn=0;
    while( ! feof(l.fd) )
      fgets(l.fd); cn++;
    end
    l.count = cn;
    l.idx = alloc( cn*sizeof(int) );
    l.allocated = cn;
    fseek(l.fd , 0, SEEK_SET);
   cn = 0;
    while( ! feof(l.fd) )
      l.idx[cn++]=ftell(l.fd);
      fgets(l.fd);
    end
    fseek(l.fd , 0, SEEK_SET);
    l.__last_idx = -2;
    l.__last = "";
    l.ready = 1;
  end
  return l.ready;
end;

/*--------------------------------------------------------------------------*/

function list_recontent( stringlist *l, string filename )
local
  int cn;
end
begin
  if( l.ready )
    list_free( l );
  end;
  list_create_fn( l, filename );
end;

/*--------------------------------------------------------------------------*/

function list_create( stringlist *l )
local
  string filename;
end
begin
  filename = list_tmpfile_randomizer( list_tmp_prefix );
  say("temp list created in: "+filename);
  l.ready = 0;
  l.fd = fopen( filename, O_WRITE );
  fclose(l.fd);
  l.fd = fopen( filename, O_READWRITE );
  if( l.fd )
    l.count = 0;
    l.idx = alloc( LIST_ALLOC_SEGMENT * sizeof(int) );
    l.allocated = LIST_ALLOC_SEGMENT;
    l.ready = 1;
    l.readonly = 0;
    l.filename = filename;
    l.__last_idx = -2;
    l.__last = "";
  else
    say("Cannot open file: "+filename);
  end
  return l.ready;
end;

/*--------------------------------------------------------------------------*/

function list_free( stringlist *l )
begin
  if( l.ready )
    fclose( l.fd );
    free( l.idx );
    if( l.filename != "" )
//      rm( l.rmfile );
    end
  end
end;

/*--------------------------------------------------------------------------*/

function string list_item(stringlist *l, int item)
begin
  if(! l.ready )
    return "*not ready*";
  end
  if( item<0 || item>=l.count )
    return "*out of bounds*";
  end
  if( item == l.__last_idx) return l.__last; end
  if( item - 1 == l.__last_idx) 
      l.__last_idx = item;
      return l.__last = fgets(l.fd); 
  end
  l.__last_idx = item;
  fseek( l.fd, l.idx[ item ], SEEK_SET );
  return l.__last = fgets(l.fd);
end;

/*--------------------------------------------------------------------------*/

function list_grow(stringlist *l, string line )
local
  int *idx2;
end
begin
  if( l.readonly ) return ; end;
  if( l.ready )
    if( l.count == l.allocated )
      say('List reealloc');
      l.allocated += LIST_ALLOC_SEGMENT;
      l.idx = realloc( l.idx , l.allocated * sizeof(int) );
    end;
  
    fseek(l.fd, 0, SEEK_END);
    l.idx[l.count] = ftell( l.fd );
    fputs( l.fd, line );
    l.count ++;
  end;
end;

/*--------------------------------------------------------------------------*/

function list_copy(stringlist *dst, stringlist *src)
local
  int i;
end
begin
  if( dst.ready )
    list_free( dst );
  end
  list_create( dst );
  
  for( i=0; i<src.count; i++ )
    list_grow( dst, list_item( src, i ) );
  end
  
end;
