#ifndef __MENU_INC_PRG__
#define __MENU_INC_PRG__ 1
/*
    Bennu Convenience Lib Set.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
#include "../lib/wm.inc.prg" // using skin settings

#include "../lib/list.inc.prg"
#include "../lib/text.inc.prg"
#include "../lib/input_module.inc.prg"

#ifndef USING_FENIX
import "mod_mem"
import "mod_draw"
#endif

/*--------------------------------------------------------------------------*/

process menu_strings(string *menu)
local
  int i,j,n;
end
begin
end;

/*--------------------------------------------------------------------------*/
// 8-bit ascii based encodings only
function word_wrap_8bit( stringlist *lst, int width)
local
  stringlist list2;
  int l1, l2, i, j, n1, n2;
  string space_chars = " ";
  string prep_chars  = ",.:;!?())";
  string eol_chars, t1, t2;
  bool found;
end
begin
  space_chars += chr(9);
  eol_chars = chr(13)+chr(10);
  
  list_create( &list2 );
  
  t1 = "";
  for( l1 = 0; l1 < lst.count; l1++ )
    frame 5;
    //say(l1+") tail:"+t1);
    t1 += list_item( lst, l1 );
    n1 = len( t1 );
  
    for( i=0; i<n1; i++ )
      if( find( eol_chars, t1[i] ) >= 0 )
	//say( substr( t1, 0, i-1 )+" |e "+substr( t1, i, n1-i ) );
	list_grow( &list2, substr( t1, 0, i-1 ) );
	t1 = substr( t1, i, n1-i ) ;
	break;
      end;
    
      if( i >= width )
	// searching for wordbreak
	found = false;
	for( ; i>0; i-- )
	  if( find( space_chars, t1[i] ) >= 0 )
	    //say( substr( t1, 0, i-1 )+" |s "+substr( t1, i, n1-i ) );
	    list_grow( &list2, substr( t1, 0, i ) );
	    found = true;
	    t1 = substr( t1, i+1, n1-i );
	    break;
	  end;
	  if( find( prep_chars, t1[i] ) >= 0 )
	    //say( substr( t1, 0, i )+" |p "+substr( t1, i, n1-i ) );
	    list_grow( &list2, substr( t1, 0, i ) );
	    t1 = substr( t1, i+1, n1-i );
	    found = true;
	    break;
	  end;
	end;
	if( found) break; end
	i = width-1;
	list_grow( &list2, substr( t1, 0, i ) );
	t1 = substr( t1, i, n1-i );
	break;	
      end;
      
    end;
    
    if( i == n1 )
      list_grow( &list2, t1 );
      t1 = "";
    end
  end;
  
  list_copy( lst, &list2 );
  list_free( &list2 );
end

/*--------------------------------------------------------------------------*/

process menu_textbox(stringlist *menu, int maxlines, int width )
local
  int i,j,n,t;
  int mw, mh, tw, th ,nowml;
  int toppos, redraw;
end
begin
  word_wrap_8bit( menu, width );
  
  graph = 0;
  
  if(!menu.ready) return; end;
  
  //effective list height in lines
  nowml = ( menu.count < maxlines ) ? menu.count : maxlines ;
  
  mw = 0;
  mh = 0;
  j  = 0;
  
  for( i=0; i < menu.count; i++)
    t = text_to_map( list_item( menu, i ) );
    tw = graphic_info( file, t, G_WIDTH);
    th = graphic_info( file, t, G_HEIGHT);
    map_unload( file, t );
    mw = tw>mw ? tw : mw;
    mh = th>mh ? th : mh;
    j += th;
  end;
  
  graph = map_new( mw + 2, mh*nowml + 2, 16 );
  set_center( file, graph, 0, 0);
  map_clear( file, graph, wm_c_bg );
  
  redraw = 1;
  loop
    
    if( ajkey( WM_KEY_UP, id ) && toppos > 0 )
      toppos --;
      redraw = 1;
    end
    
    if( ajkey( WM_KEY_DOWN, id ) && (toppos + nowml) < menu.count )
      toppos ++;
      redraw = 1;
    end
    
    if( ajkey( WM_KEY_LEFT, id ) && (toppos - maxlines) > 0 )
      toppos -= maxlines;
      redraw = 1;
    end
    
    if( ajkey( WM_KEY_RIGHT, id ) && (toppos + nowml + maxlines) < menu.count )
      toppos += maxlines;
      redraw = 1;
    end
    
    if( redraw )
      map_clear( file, graph, wm_c_bg );
      th = 0;
      set_text_color( wm_c_fg );
      for( i=toppos, j=0 ; j < nowml && i < menu.count; i++, j++ )
	t = text_to_map( list_item( menu, i ) );
	map_put( file, graph, t, 1, th+1 );
	th += graphic_info( file, t, G_HEIGHT);
	map_unload( file, t );
      end;
      redraw = 0;
      x = 1; // update flag
    end
    
    frame;
  end;
  
onexit
  if( graph )
    map_unload( file, graph );
  end;
end;

/*--------------------------------------------------------------------------*/

process menu_selectbox(stringlist *menu, int maxlines)
public
  int rewidth;   /// recount width
  int selected;  /// index of selected item
local
  int i,j,n,t;
  int jk, waitk, nowml;
  int mw, mh, tw, th;
  int toppos, redraw;
end
begin

  graph = 0;
  selected = 0;
  
  if(!menu.ready) return; end;

  //effective list height in lines
  nowml = ( menu.count < maxlines ) ? menu.count : maxlines ;
    
  mw = 0;
  mh = 0;
  j  = 0;
  
  for( i=0; i < menu.count; i++)
    t = text_to_map( list_item( menu, i ) );
    tw = graphic_info( file, t, G_WIDTH);
    th = graphic_info( file, t, G_HEIGHT);
    map_unload( file, t );
    mw = tw>mw ? tw : mw;
    mh = th>mh ? th : mh;
    j += th;
  end;
  
  graph = map_new( mw+2, mh*nowml + 2, 16 );
  set_center( file, graph, 0, 0);
  map_clear( file, graph, wm_c_bg );
  
  redraw = 1;
  jk = _JKEY_LAST;
  waitk = 0;
  rewidth = 0;
  loop
      
    if( ajkey( WM_KEY_UP, id ) && selected  > 0 )
      selected --;
      redraw = 1;
      waitk = jk == _JKEY_LAST;
      jk = WM_KEY_UP;
    end
    
    if( ajkey( WM_KEY_DOWN, id ) && selected < menu.count -1 )
      selected ++;
      redraw = 1;
      waitk = jk == _JKEY_LAST;
      jk = WM_KEY_DOWN;
    end
    
    if( rewidth )     
      mw = 0;
      mh = 0;
      j  = 0;
      nowml = ( menu.count < maxlines ) ? menu.count : maxlines ;
      for( i=0; i < menu.count; i++)
	//say('REWIDTH: '+list_item( menu, i ));
	t = text_to_map( list_item( menu, i ) );
	tw = graphic_info( file, t, G_WIDTH);
	th = graphic_info( file, t, G_HEIGHT);
	map_unload( file, t );
	mw = tw>mw ? tw : mw;
	mh = th>mh ? th : mh;
	j += th;
      end;
      map_unload( file, graph );
      graph = map_new( mw+2, mh*nowml + 2, 16 );
      set_center( file, graph, 0, 0);
      map_clear( file, graph, wm_c_bg );
      rewidth = 0;
      x = 1;
    end
    
    if( redraw )     
      map_clear( file, graph, wm_c_bg );
      th = 0;
      if( selected >= toppos + nowml ) toppos = selected - nowml + 1; end;
      if( selected < toppos ) toppos = selected; end;
      for( i=toppos, j=0 ; j < nowml && i < menu.count; i++, j++ )
	if(i == selected )
	  set_text_color ( wm_c_bg );
	  t = text_to_map( list_item( menu, i ) );
	
	  drawing_map(file, graph );
	  drawing_color( wm_c_fg );
	  drawing_alpha( 255 );
	  draw_box( 1, th + 1, mw , th + graphic_info( file, t, G_HEIGHT ) );
	  map_put( file, graph, t, 1, th+1 );
	else
	  set_text_color ( wm_c_fg );
	  t = text_to_map( list_item( menu, i ) );
	  map_put( file, graph, t, 1, th+1 );
	end
	th += graphic_info( file, t, G_HEIGHT );
	map_unload( file, t );      
      end;    
      redraw = 0;
      x = 1; // update flag
      j = 5; // frames to wait before keyrepeat;
      while ( waitk && j-- &&  ajkey( jk, id ) ) frame; end;
    else
      jk = _JKEY_LAST;
    end
    
    frame;
  end;
  
onexit
  if( graph )
    map_unload( file, graph );
  end;
end;

/*--------------------------------------------------------------------------*/

process menu_progressbar( int minvalue, int maxvalue, int *value )
public
  int prev, ww;
end
begin
  graph = map_new( ww = maxvalue - minvalue, wm_size , 16);
  prev = (*value) -1 ;
  
  loop
    if( prev != (*value) )
      map_clear( file, graph, wm_c_bg );
      drawing_map( file, graph );
      drawing_color( wm_c_fg );
      draw_box( 1, 1, ( ww-2 ) * (*value) / ww  , wm_size-2 );
      x++;
    end
    prev = (*value);
    frame;
  end;
  
onexit
  if( graph )
    map_unload( file, graph );
  end
end;

#endif
