#ifndef __SYSOPTS_INC__
#define __SYSOPTS_INC__
/*
    Bennu Convenience Lib Set.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

/**
 *  This is global link over components
 *  and conversion of OS_CODES to Y/N capabilities
 * 
 *  This file needs to be updated whenever Bennu gains 
 */

global 
STRUCT __sysopts_struct_
  bool _initflag = false;
  
  /**
   * Default rasters bpp.
   */
  int default_bpp;
  
  /**
   * Is device a handheld of GP2x, Wiz, Caanoo or PSP type
   */
  int is_handheld;
  
  /**
   * keyboard of device is normally dead
   * and cannot be simply accessed (non-pc devices)
   */
  int dead_keyboard;
  
  /**
   * Device has net access
   */
  int has_net;
  
  /**
   * Device has access to basic posix cli utils (ls, cat, tar etc.)
   */
  int has_posix_cli;
  
   /**
   * Device controlled by Linux kernel 2.6
   * (needed for cpuload)
   */
  int is_linux_26; 
  
  /**
   * screen caps
   * 
   */
  int scr_rel_h;
  int scr_rel_w;
END;
////////////////////////////////////////////////////////////////

function sys_opts_init()
begin

__sysopts_struct_.is_handheld = (os_id == OS_GP32) || (os_id == OS_GP2X) || (os_id == OS_GP2X_WIZ) || (os_id == OS_CAANOO);

__sysopts_struct_.dead_keyboard = !( (os_id == OS_WIN32) || (os_id == OS_LINUX) || (os_id == OS_BEOS) || (os_id == OS_MACOS) || (os_id == OS_BSD) );

// No any net support yet
__sysopts_struct_.has_net = 0;

__sysopts_struct_.has_posix_cli = (os_id == OS_LINUX) || (os_id == OS_BEOS) || (os_id == OS_MACOS) || (os_id == OS_BSD) ||  (os_id == OS_GP2X) || (os_id == OS_GP2X_WIZ) || (os_id == OS_CAANOO);
__sysopts_struct_.is_linux_26   = (os_id == OS_LINUX) || (os_id == OS_GP2X_WIZ) || (os_id == OS_CAANOO);

__sysopts_struct_.default_bpp = __sysopts_struct_.is_handheld ? 16 : 32;

__sysopts_struct_._initflag = true;

end

////////////////////////////////////////////////////////////////
function int sys_is_handheld()
begin
  if (!__sysopts_struct_._initflag ) sys_opts_init(); end;
  return __sysopts_struct_.is_handheld ;
end;

function int sys_dead_keyboard()
begin
  if (!__sysopts_struct_._initflag ) sys_opts_init(); end;
  return __sysopts_struct_.dead_keyboard ;
end;

function int sys_has_net()
begin
  if (!__sysopts_struct_._initflag ) sys_opts_init(); end;
  return __sysopts_struct_.has_net ;
end;

function int sys_has_posix_cli()
begin
  if (!__sysopts_struct_._initflag ) sys_opts_init(); end;
  return __sysopts_struct_.has_posix_cli ;
end;

function int sys_is_linux_26()
begin
  if (!__sysopts_struct_._initflag ) sys_opts_init(); end;
  return __sysopts_struct_.is_linux_26 ;
end;

function int sys_default_bpp()
begin
  if (!__sysopts_struct_._initflag ) sys_opts_init(); end;
  return __sysopts_struct_.default_bpp ;
end;

#endif