#ifndef __TEXT_INC_PRG__
#define __TEXT_INC_PRG__
/*
    Bennu Convenience Lib Set.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

#ifndef USING_FENIX
import "mod_text"
#endif

#define TXT_RENDER_F_NATIVE 0
#define TXT_RENDER_F_CFNT 1
       
#include "../lib/cfnt_font.inc.prg"
       
global
  int txt_render_frontend = TXT_RENDER_F_NATIVE;
  
  string txt_render_opt_cfnt_encpath="../res/encodings/";
  string txt_render_opt_cfnt_fontpath="../res/fonts/";
  
  CFNT_FONT txt_render_opt_cfnt_font;
end

//
// FRONTEND-SPECIFIC FUNCTIONS
//
/*--------------------------------------------------------------------------------*/
function text_set_enc_cfnt( string encname );
begin
  cfnt_add_encoding_from_file( &txt_render_opt_cfnt_font, txt_render_opt_cfnt_encpath + encname + ".enc" );
end
/*--------------------------------------------------------------------------------*/
function int text_set_font_cfnt( string fontname );
begin
  if( fexists(txt_render_opt_cfnt_fontpath + fontname + ".psf") )
    return cfnt_load_psf( &txt_render_opt_cfnt_font, txt_render_opt_cfnt_fontpath + fontname + ".psf" );
  end
  return cfnt_load_raw( &txt_render_opt_cfnt_font, txt_render_opt_cfnt_fontpath + fontname + ".raw", 14);
end
/*--------------------------------------------------------------------------------*/
function text_init_cfnt();
begin
  text_set_font_cfnt( "ter-c14n" );
  text_set_enc_cfnt( "microsoft-cp1251" );
end
/*--------------------------------------------------------------------------------*/
function text_free_cfnt();
begin
  cfnt_unload( &txt_render_opt_cfnt_font );
end
/*--------------------------------------------------------------------------------*/

// 
//  ABSTRACT FUNCTIONS
//
/*--------------------------------------------------------------------------------*/
function text_set_encoding(string encname)
begin
  if( txt_render_frontend == TXT_RENDER_F_NATIVE )
    return;
  end;
  if( txt_render_frontend == TXT_RENDER_F_CFNT )
    text_set_enc_cfnt( encname );
  end;
end

/*--------------------------------------------------------------------------------*/

function text_set_font(string fontname)
begin
  if( txt_render_frontend == TXT_RENDER_F_NATIVE )
    return;
  end;
  if( txt_render_frontend == TXT_RENDER_F_CFNT )
    text_set_font_cfnt( fontname );
  end;
end
/*--------------------------------------------------------------------------------*/

function text_init_frontend()
begin
  if( txt_render_frontend == TXT_RENDER_F_NATIVE )
    return;
  end;
  if( txt_render_frontend == TXT_RENDER_F_CFNT )
    text_init_cfnt();
  end;
end
/*--------------------------------------------------------------------------------*/

function text_free_frontend()
begin
  if( txt_render_frontend == TXT_RENDER_F_NATIVE )
    return;
  end;
  if( txt_render_frontend == TXT_RENDER_F_CFNT )
    text_free_cfnt();
  end;
end

/*--------------------------------------------------------------------------------*/

function text_set_frontend(int fe)
begin
  text_free_frontend();
  txt_render_frontend = fe;
  text_init_frontend();
  say( "Font frontend set: " + fe );
end

/*--------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------*/

function text_to_map( string atext )
local
  int map, cmap;
end
begin
  if( txt_render_frontend == TXT_RENDER_F_NATIVE )
    if( atext == "" )
      // a feature of write_in_map(), it cannot write emty strings correctly
      return write_in_map( txt_render_opt_cfnt_font.native_analog, " " ,ALIGN_TOP_LEFT );
    else
      return write_in_map( txt_render_opt_cfnt_font.native_analog, atext ,ALIGN_TOP_LEFT );
    end;
  end;
  if( txt_render_frontend == TXT_RENDER_F_CFNT )
    map  = map_new( txt_render_opt_cfnt_font.width * len(atext), txt_render_opt_cfnt_font.height, 16 /*sys_default_bpp()*/ );
    cmap = map_new( txt_render_opt_cfnt_font.width * len(atext), txt_render_opt_cfnt_font.height, 16 /*sys_default_bpp()*/ );
    map_clear( file, cmap, get_text_color() );
    set_center( file, map, 0, 0 );
    set_center( file, cmap, 0, 0 );
    cfnt_write_to_map( &txt_render_opt_cfnt_font, map, 0, 0, atext );
    map_xput( file, map, cmap, 0, 0, 0, 100, B_SBLEND );
    map_unload( file, cmap );
    return map;
  end;
  return 0;
end;

#endif
